@@ -3,7 +3,7 @@ use warnings;
 use 5.008;
 
 package JSON::Path::Constants;
-$JSON::Path::Constants::VERSION = '0.310';
+$JSON::Path::Constants::VERSION = '0.400';
 # ABSTRACT: Constants used in the JSON::Path distribution
 
 # VERSION
@@ -13,10 +13,11 @@ use Readonly;
 use Exporter::Easy (
     TAGS => [
         symbols => [
-            '$DOLLAR_SIGN',          '$COMMERCIAL_AT',     '$FULL_STOP',     '$LEFT_SQUARE_BRACKET',
-            '$RIGHT_SQUARE_BRACKET', '$ASTERISK',          '$COLON',         '$LEFT_PARENTHESIS',
-            '$RIGHT_PARENTHESIS',    '$COMMA',             '$QUESTION_MARK', '$EQUAL_SIGN',
-            '$EXCLAMATION_MARK',     '$GREATER_THAN_SIGN', '$LESS_THAN_SIGN',
+            '$DOLLAR_SIGN',          '$COMMERCIAL_AT',     '$FULL_STOP',      '$LEFT_SQUARE_BRACKET',
+            '$RIGHT_SQUARE_BRACKET', '$ASTERISK',          '$COLON',          '$LEFT_PARENTHESIS',
+            '$RIGHT_PARENTHESIS',    '$COMMA',             '$QUESTION_MARK',  '$EQUAL_SIGN',
+            '$EXCLAMATION_MARK',     '$GREATER_THAN_SIGN', '$LESS_THAN_SIGN', '$QUOTATION_MARK',
+            '$APOSTROPHE'
         ],
         operators => [
             '$TOKEN_ROOT',           '$TOKEN_CURRENT',
@@ -33,6 +34,8 @@ use Exporter::Easy (
     ]
 );
 
+Readonly our $QUOTATION_MARK       => q{"};
+Readonly our $APOSTROPHE           => q{'};
 Readonly our $DOLLAR_SIGN          => '$';
 Readonly our $COMMERCIAL_AT        => '@';
 Readonly our $FULL_STOP            => '.';
@@ -84,7 +87,7 @@ JSON::Path::Constants - Constants used in the JSON::Path distribution
 
 =head1 VERSION
 
-version 0.310
+version 0.400
 
 =head1 AUTHOR
 
